
/*
ksubsetselector.cpp - implementation file for the KSubSetSelector class.

Terms and Conditions

Copyright (c) 2001 Catalin Climov <xxl@thekompany.com>

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
COPYRIGHT HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of the copyright holder shall not
be used in advertising or otherwise to promote the sale, use or other dealings
in this Software without prior written authorization from the copyright holder.
*/

#include "ksubsetselector.h"
#include "arrowdata.h"

#include <klistbox.h>
#include <qframe.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>

/*
 *  Constructs a KSubsetSelector which is a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'
 */
KSubsetSelector::KSubsetSelector( QWidget* parent,  const char* name, WFlags fl )
    : QWidget( parent, name, fl )
{
    setupUI();
    setupConnections();
}

/*
 *  Destroys the object and frees any allocated resources
 */
KSubsetSelector::~KSubsetSelector()
{
    // no need to delete child widgets, Qt does it all for us
}

void KSubsetSelector::setupUI()
{
    QPixmap leftPix( ( const char** ) left_pix_data );
    QPixmap upPix( ( const char** ) up_pix_data );
    QPixmap rightPix( ( const char** ) right_pix_data );
    QPixmap downPix( ( const char** ) down_pix_data );

    QGridLayout* layout = new QGridLayout( this );
    layout->setSpacing( 6 );
    layout->setMargin( 11 );

    QLabel* label = new QLabel( this );
    label->setText( tr( "KwiX Button Layout" ) );
    layout->addMultiCellWidget( label, 0, 0, 0, 2 );

    QFrame* line = new QFrame( this );
    line->setFrameStyle( QFrame::HLine | QFrame::Sunken );
    layout->addMultiCellWidget( line, 1, 1, 0, 2 );

    label = new QLabel( this );
    label->setText( tr( "Available:" ) );
    layout->addWidget( label, 2, 0);

    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Minimum );
    layout->addItem( spacer, 2, 1 );

    label = new QLabel( this );
    label->setText( tr( "Current:" ) );
    layout->addWidget( label, 2, 2);

    spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    layout->addItem( spacer, 3, 1 );

    QFrame* arrowPad = new QFrame( this );
    arrowPad->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)1, arrowPad->sizePolicy().hasHeightForWidth() ) );
    arrowPad->setFrameShape( QFrame::NoFrame );
    arrowPad->setFrameShadow( QFrame::Raised );
    QGridLayout* padLayout = new QGridLayout( arrowPad );
    padLayout->setSpacing( 0 );
    padLayout->setMargin( 0 );

    _leftArrow = new QToolButton( arrowPad );
    _leftArrow->setPixmap( leftPix );
    padLayout->addWidget( _leftArrow, 1, 0 );

    _upArrow = new QToolButton( arrowPad );
    _upArrow->setPixmap( upPix );
    padLayout->addWidget( _upArrow, 0, 1 );

    _rightArrow = new QToolButton( arrowPad );
    _rightArrow->setPixmap( rightPix );
    padLayout->addWidget( _rightArrow, 1, 2 );

    _downArrow = new QToolButton( arrowPad );
    _downArrow->setPixmap( downPix );
    padLayout->addWidget( _downArrow, 2, 1 );

    layout->addWidget( arrowPad, 4, 1 );

    spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    layout->addItem( spacer, 5, 1 );

    _leftBox = new KListBox( this );
    layout->addMultiCellWidget( _leftBox, 3, 5, 0, 0 );

    _rightBox = new KListBox( this );
    layout->addMultiCellWidget( _rightBox, 3, 5, 2, 2 );

    _leftArrow->setEnabled(false);
    _rightArrow->setEnabled(false);
    _upArrow->setEnabled(false);
    _downArrow->setEnabled(false);

//    _leftBox->insertItem("One");
//    _leftBox->insertItem("Two");
//    _leftBox->insertItem("Three");
}

void KSubsetSelector::setupConnections()
{
    connect(_leftBox, SIGNAL(selectionChanged()), SLOT(slotLeftSelectionChanged()));
    connect(_rightBox, SIGNAL(selectionChanged()), SLOT(slotRightSelectionChanged()));
    connect(_rightArrow, SIGNAL(clicked()), SLOT(slotRightArrowClicked()));
    connect(_leftArrow, SIGNAL(clicked()), SLOT(slotLeftArrowClicked()));
    connect(_upArrow, SIGNAL(clicked()), SLOT(slotUpArrowClicked()));
    connect(_downArrow, SIGNAL(clicked()), SLOT(slotDownArrowClicked()));
    connect(_leftBox, SIGNAL(highlighted(int)), SLOT(slotLeftHighlighted(int)));
    connect(_rightBox, SIGNAL(highlighted(int)), SLOT(slotRightHighlighted(int)));
}

void KSubsetSelector::slotLeftSelectionChanged()
{
    bool hasSelection = _leftBox->currentItem() != -1;
    _rightArrow->setEnabled(hasSelection);
}

void KSubsetSelector::slotRightSelectionChanged()
{
    int item = _rightBox->currentItem();
    bool hasSelection = item != -1;
    bool alwaysSelected = false;
    if( hasSelection )
        alwaysSelected = dynamic_cast<KSubsetElement*>(_rightBox->item(item))->alwaysSelected();
    _leftArrow->setEnabled(hasSelection && !alwaysSelected );
    _upArrow->setEnabled(hasSelection && item > 0);
    _downArrow->setEnabled(hasSelection && item < (int)_rightBox->count()-1 );
}

void KSubsetSelector::slotRightArrowClicked()
{
    int item = _leftBox->currentItem();
    // paranoia
    if( item != -1 )
    {
        KSubsetElement* element = dynamic_cast<KSubsetElement*>(_leftBox->item(item));
        _rightBox->insertItem(new KSubsetElement(element));
        if( ! element->multi() )
            _leftBox->removeItem(item);
        emit selectionChanged();
        emit selectionModified();
    }
    slotLeftSelectionChanged();
}

void KSubsetSelector::slotLeftArrowClicked()
{
    int item = _rightBox->currentItem();
    // paranoia
    if( item != -1 )
    {
        KSubsetElement* element = dynamic_cast<KSubsetElement*>(_rightBox->item(item));
        if( ! element->multi() )
        {
            int i = 0;
            while( i<(int)_leftBox->count() && dynamic_cast<KSubsetElement*>(_leftBox->item(i))->id() < element->id())
                i++;
            _leftBox->insertItem(new KSubsetElement(element),i);
        }
        _rightBox->removeItem(item);
        emit selectionChanged();
        emit selectionModified();
    }
    slotRightSelectionChanged();
}

void KSubsetSelector::slotUpArrowClicked()
{
    int item = _rightBox->currentItem();
    // paranoia
    if( item > 0 )
    {
        _rightBox->insertItem(new KSubsetElement(dynamic_cast<KSubsetElement*>(_rightBox->item(item))),item-1);
        _rightBox->removeItem(item+1);
        _rightBox->setCurrentItem(item-1);
        emit orderChanged();
        emit selectionModified();
    }
    slotRightSelectionChanged();
}

void KSubsetSelector::slotDownArrowClicked()
{
    int item = _rightBox->currentItem();
    // paranoia
    if( item < (int)_rightBox->count()-1 )
    {
        _rightBox->insertItem(new KSubsetElement(dynamic_cast<KSubsetElement*>(_rightBox->item(item))),item+2);
        _rightBox->removeItem(item);
        _rightBox->setCurrentItem(item+1);
        emit orderChanged();
        emit selectionModified();
    }
    slotRightSelectionChanged();
}

void KSubsetSelector::slotLeftHighlighted(int item)
{
    _leftBox->setCurrentItem(item);
}

void KSubsetSelector::slotRightHighlighted(int item)
{
    _rightBox->setCurrentItem(item);
}

void KSubsetSelector::setElements( const KSubset& full, const QArray<int>& selected )
{
    _full = full;
    _selected = selected;
    _leftBox->clear();
    _rightBox->clear();
    for(QListIterator<KSubsetElement> it(_full); it.current(); ++it)
        _leftBox->insertItem(new KSubsetElement(it.current()));
    for(int i = 0; i < (int)_selected.size(); i++)
        for(QListIterator<KSubsetElement> it(_full); it.current(); ++it)
            if( _selected[i] == it.current()->id() )
            {
                _rightBox->insertItem(new KSubsetElement(it.current()));
                if( ! it.current()->multi() )
                    for(int j = 0; j < (int)_leftBox->count(); j++)
                        if( dynamic_cast<KSubsetElement*>(_leftBox->item(j))->id() == _selected[i] )
                            _leftBox->removeItem(j);
            }
}

const QArray<int> KSubsetSelector::selectedElements()
{
    QArray<int> ret(_rightBox->count());
    for( int i = 0; i < (int)_rightBox->count(); i++ )
        ret[i] = dynamic_cast<KSubsetElement*>(_rightBox->item(i))->id();
    return ret;
}
